"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Type_1 = require("../Type");
var Strings_1 = require("../Strings");
var Source_1 = require("../Source");
var RendererOptions_1 = require("../RendererOptions");
var JavaScript_1 = require("./JavaScript");
var TypeScriptTargetLanguage = /** @class */ (function (_super) {
    __extends(TypeScriptTargetLanguage, _super);
    function TypeScriptTargetLanguage() {
        var _this = _super.call(this, "TypeScript", ["typescript", "ts", "tsx"], "ts") || this;
        _this._justTypes = new RendererOptions_1.BooleanOption("just-types", "Interfaces only", false);
        _this._declareUnions = new RendererOptions_1.BooleanOption("explicit-unions", "Explicitly name unions", false);
        return _this;
    }
    TypeScriptTargetLanguage.prototype.getOptions = function () {
        return [this._justTypes, this._declareUnions, this.omitRuntimeTypecheck];
    };
    Object.defineProperty(TypeScriptTargetLanguage.prototype, "rendererClass", {
        get: function () {
            return TypeScriptRenderer;
        },
        enumerable: true,
        configurable: true
    });
    return TypeScriptTargetLanguage;
}(JavaScript_1.JavaScriptTargetLanguage));
exports.TypeScriptTargetLanguage = TypeScriptTargetLanguage;
var TypeScriptRenderer = /** @class */ (function (_super) {
    __extends(TypeScriptRenderer, _super);
    function TypeScriptRenderer(graph, leadingComments, _justTypes, declareUnions, omitRuntimeTypecheck) {
        var _this = _super.call(this, graph, leadingComments, omitRuntimeTypecheck) || this;
        _this._justTypes = _justTypes;
        _this._inlineUnions = !declareUnions;
        return _this;
    }
    TypeScriptRenderer.prototype.sourceFor = function (t) {
        var _this = this;
        return Type_1.matchType(t, function (_anyType) { return Source_1.singleWord("any"); }, function (_nullType) { return Source_1.singleWord("null"); }, function (_boolType) { return Source_1.singleWord("boolean"); }, function (_integerType) { return Source_1.singleWord("number"); }, function (_doubleType) { return Source_1.singleWord("number"); }, function (_stringType) { return Source_1.singleWord("string"); }, function (arrayType) {
            var itemType = _this.sourceFor(arrayType.items);
            if ((arrayType.items instanceof Type_1.UnionType && _this._inlineUnions) ||
                arrayType.items instanceof Type_1.ArrayType) {
                return Source_1.singleWord(["Array<", itemType.source, ">"]);
            }
            else {
                return Source_1.singleWord([Source_1.parenIfNeeded(itemType), "[]"]);
            }
        }, function (classType) { return Source_1.singleWord(_this.nameForNamedType(classType)); }, function (mapType) { return Source_1.singleWord(["{ [key: string]: ", _this.sourceFor(mapType.values).source, " }"]); }, function (enumType) { return Source_1.singleWord(_this.nameForNamedType(enumType)); }, function (unionType) {
            if (_this._inlineUnions || Type_1.nullableFromUnion(unionType) !== null) {
                var children = unionType.children.map(function (c) { return Source_1.parenIfNeeded(_this.sourceFor(c)); });
                return Source_1.multiWord.apply(void 0, [" | "].concat(children.toArray()));
            }
            else {
                return Source_1.singleWord(_this.nameForNamedType(unionType));
            }
        });
    };
    TypeScriptRenderer.prototype.emitEnum = function (e, enumName) {
        var _this = this;
        this.emitDescription(this.descriptionForType(e));
        this.emitBlock(["export enum ", enumName], "", function () {
            _this.forEachEnumCase(e, "none", function (name, jsonName) {
                _this.emitLine(name, " = \"" + Strings_1.utf16StringEscape(jsonName) + "\",");
            });
        });
    };
    TypeScriptRenderer.prototype.emitClass = function (c, className) {
        var _this = this;
        this.emitDescription(this.descriptionForType(c));
        this.emitBlock(["export interface ", className], "", function () {
            _this.emitPropertyTable(c, function (name, _jsonName, p) {
                var t = p.type;
                var nullable = t instanceof Type_1.UnionType ? Type_1.nullableFromUnion(t) : null;
                if (p.isOptional && nullable === null) {
                    nullable = t;
                }
                return [
                    [name, nullable !== null ? "?" : "", ": "],
                    [_this.sourceFor(nullable !== null ? nullable : t).source, ";"]
                ];
            });
        });
    };
    TypeScriptRenderer.prototype.emitUnion = function (u, unionName) {
        var _this = this;
        if (this._inlineUnions) {
            return;
        }
        this.emitDescription(this.descriptionForType(u));
        var children = Source_1.multiWord.apply(void 0, [" | "].concat(u.children.map(function (c) { return Source_1.parenIfNeeded(_this.sourceFor(c)); }).toArray()));
        this.emitLine("export type ", unionName, " = ", children.source, ";");
    };
    TypeScriptRenderer.prototype.emitTypes = function () {
        var _this = this;
        this.forEachNamedType("leading-and-interposing", function (c, n) { return _this.emitClass(c, n); }, function (e, n) { return _this.emitEnum(e, n); }, function (u, n) { return _this.emitUnion(u, n); });
    };
    TypeScriptRenderer.prototype.emitUsageImportComment = function () {
        var topLevelNames = [];
        this.forEachTopLevel("none", function (_t, name) {
            topLevelNames.push(", ", name);
        }, Type_1.isNamedType);
        this.emitLine("//   import { Convert", topLevelNames, ' } from "./file";');
    };
    TypeScriptRenderer.prototype.emitUsageComments = function () {
        if (this._justTypes)
            return;
        _super.prototype.emitUsageComments.call(this);
    };
    TypeScriptRenderer.prototype.deserializerFunctionLine = function (t, name) {
        return ["export function to", name, "(json: string): ", this.sourceFor(t).source];
    };
    TypeScriptRenderer.prototype.serializerFunctionLine = function (t, name) {
        var camelCaseName = Source_1.modifySource(Strings_1.camelCase, name);
        return ["export function ", camelCaseName, "ToJson(value: ", this.sourceFor(t).source, "): string"];
    };
    Object.defineProperty(TypeScriptRenderer.prototype, "moduleLine", {
        get: function () {
            return "export module Convert";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TypeScriptRenderer.prototype, "castFunctionLine", {
        get: function () {
            return "function cast<T>(obj: any, typ: any): T";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TypeScriptRenderer.prototype, "typeAnnotations", {
        get: function () {
            return { any: ": any", anyArray: ": any[]", string: ": string", boolean: ": boolean" };
        },
        enumerable: true,
        configurable: true
    });
    TypeScriptRenderer.prototype.emitModuleExports = function () {
        return;
    };
    TypeScriptRenderer.prototype.emitConvertModule = function () {
        if (this._justTypes)
            return;
        _super.prototype.emitConvertModule.call(this);
    };
    return TypeScriptRenderer;
}(JavaScript_1.JavaScriptRenderer));
